#ifndef _PROG_H_
#define _PROG_H_
#include <stdio.h>
#include <ctype.h>
#include "defs.h"
#include "video.h"
#include "_tape.h"
#include "utils.h"
#include "state.h"
#include "clock.h"

#define MAX_LINES  99

struct TLine {
  char from[12];
  int command;
  char to[12];
  int is_break;
} lines[MAX_LINES];

class TProgram {
    void Draw();
    char filename[40];
    int pos_y;
    int e1, e3;
    TState *state;
    int cur_rule;
    long cur_step;
    char attr1, attr2, attr3;
    void UpdateFilename();
  public:
    TProgram();
    ~TProgram();
    void Clear();
    void Update();
    void Edit(unsigned char, char);
    int Run();
    void Reset();
    void Save();
    void Load();
    void SaveTape(char *);
    void LoadTape(char *);
    void ClearBreakPoints();
};

TProgram::TProgram()
{
  state = new TState(56, 5);
  strcpy(filename, "NONAME.MAK");
  cur_rule = 0;
  cur_step = 0;
  Draw();
  Clear();
}

TProgram::~TProgram()
{
  delete state;
}

void TProgram::Draw()
{
  fill(4, 5, 38, 1, '', PROGRAM_COLOR);
  fill(4, 16, 38, 1, '', PROGRAM_COLOR);
  fill(3, 6, 1, 10, '', PROGRAM_COLOR);
  fill(42, 6, 1, 10, '', PROGRAM_COLOR);
  out_char(3, 5, '', PROGRAM_COLOR);
  out_char(42, 5, '', PROGRAM_COLOR);
  out_char(3, 16, '', PROGRAM_COLOR);
  out_char(42, 16, '', PROGRAM_COLOR);
  fill(4, 6, 38, 10, ' ', PROGRAM_COLOR);
  set_attr(5, 17, 40, 1, SHADOW_COLOR);
  set_attr(43, 6, 2, 12, SHADOW_COLOR);
  out_char(10, 5, '', PROGRAM_COLOR);
  fill(10, 6, 1, 10, '', PROGRAM_COLOR);
  out_char(10, 16, '', PROGRAM_COLOR);

  for (int i = 0; i < 2; i++)
  {
    out_char(23 + i * 6, 5, '', PROGRAM_COLOR);
    fill(23 + i * 6, 6, 1, 10, '', PROGRAM_COLOR);
    out_char(23 + i * 6, 16, '', PROGRAM_COLOR);
  }

  UpdateFilename();
}

void TProgram::Clear()
{
  pos_y = 0;
  e1 = 0;
  e3 = 0;
  for (int i = 0; i < MAX_LINES; i++)
  {
    lines[i].is_break = 0;
    lines[i].command = 0;
    lines[i].from[0] = 0;
    lines[i].to[0] = 0;
  }
  Update();
}

void TProgram::Update()
{
  char buf[10];

  for (int i = 0; i < 10; i++)
  {
    attr1 = PROGRAM_SYMCOLOR;
    attr2 = PROGRAM_COMMANDCOLOR;
    attr3 = attr1;
    if (lines[i + pos_y].command == 2)
    {
      attr1 = attr2 = attr3 = PROGRAM_STOPCOLOR;
    }
    if (i == e1)
    {
      switch (e3)
      {
        case 0: attr1 = EDIT_TEXTCOLOR; gotoxy(13 + strlen(lines[i + pos_y].from), i + 7); break;
        case 1: attr2 = EDIT_TEXTCOLOR; gotoxy(26, i + 7); break;
        case 2: attr3 = EDIT_TEXTCOLOR; gotoxy(32 + strlen(lines[i + pos_y].to), i + 7); break;
      }
    }
    sprintf(buf, "%2d", i + pos_y + 1);
    out_string(6, i + 6, buf, PROGRAM_TEXTCOLOR);
    out_string(12, i + 6, lines[i + pos_y].from, attr1);
    fill(12 + strlen(lines[i + pos_y].from), i + 6, 10 - strlen(lines[i + pos_y].from), 1, ' ', attr1);
    switch (lines[i + pos_y].command)
    {
      case 0: fill(25, i + 6, 3, 1, ' ', attr2); break;
      case 1: out_string(25, i + 6, "-->", attr2); break;
      case 2: out_string(25, i + 6, "|->", attr2); break;
    }
    out_string(31, i + 6, lines[i + pos_y].to, attr3);
    fill(31 + strlen(lines[i + pos_y].to), i + 6, 10 - strlen(lines[i + pos_y].to), 1, ' ', attr3);
    if (lines[i + pos_y].is_break)
    {
      out_char(4, i + 6, '', 0x34);
      out_char(41, i + 6, '', 0x34);
    }
    else
    {
      out_char(4, i + 6, ' ', PROGRAM_TEXTCOLOR);
      out_char(41, i + 6, ' ', PROGRAM_TEXTCOLOR);
    }
  }
}

void TProgram::Edit(unsigned char command, char key)
{
  int x, y, a;

  switch (command)
  {
    case 75: if (e3) e3--; break;
    case 77: if (e3 < 2) e3++; break;
    case 72: if (e1)
               e1--;
             else
             {
               if (pos_y)
                 pos_y--;
             }
             break;
    case 80: if (e1 < 9)
               e1++;
             else
             {
               if (pos_y < MAX_LINES - 10)
                 pos_y++;
             }
             break;
    case 83: switch (e3)
             {
               case 0: lines[e1 + pos_y].from[0] = 0; break;
               case 1: lines[e1 + pos_y].command = 0; break;
               case 2: lines[e1 + pos_y].to[0] = 0; break;
             }
             break;
    case 255: lines[e1 + pos_y].is_break ^= 1;
              Update();
              return;
  }

  char *t;
  if (!e3)
    t = lines[e1 + pos_y].from;
  else if (e3 == 2)
    t = lines[e1 + pos_y].to;

  if (key >= 0x20)
  {
    if (strlen(t) < 9)
    {
      while (*t++);
      *t-- = 0;
      *t = key;
    }
  }
  if (key == 8)
  {
    if (strlen(t))
    {
      while (*t++);
      t -= 2;
      *t = 0;
    }
    else
    {
      for (int i = e1 + pos_y; i < MAX_LINES; i++)
        if (i < MAX_LINES + 1)
          lines[i] = lines[i + 1];
        else
        {
          lines[i].from[0] = 0;
          lines[i].to[0] = 0;
          lines[i].command = 0;
        }
    }
  }
  if ((key == 9) && (e3 == 1))
  {
    lines[e1 + pos_y].command++;
    if (lines[e1 + pos_y].command > 2)
      lines[e1 + pos_y].command = 1;
  }
  if ((key == 13) && (e1 + pos_y < MAX_LINES - 1))
  {
    for (int i = MAX_LINES - 1; i > e1 + pos_y; i--)
      lines[i] = lines[i - 1];
  }

  Update();
}

int TProgram::Run()
{
  int i = 0, res;
  do
  {
    res = tape -> Search(lines[i].from);
  } while((lines[i++].command) && (res < 0));
  i--;
  if (res < 0) res = 0;
  if (lines[i].command)
  {
    cur_rule = i;
    if (strlen(lines[i].to) + strlen(tape -> data) > 999)
    {
      Reset();
      MessageBox("Error", "Tape is full\n", 0);
      return 0;
    }
    tape -> Delete(res, strlen(lines[i].from));
    tape -> Insert(res, lines[i].to);
    tape -> Edit(71, 0);
  }
  else
    cur_rule = -1;
  cur_step++;
  state -> Update(cur_rule + 1, cur_step, lines[i].from, lines[i].to);
  if ((lines[cur_rule].command == 2) || (cur_rule < 0))
  {
    if (cur_rule < 0)
      MessageBox("Information", "Program complete -\nno matching rules found\n", 0);
    else
      MessageBox("Information", "Program complete -\ntermination rule used\n", 0);
    Reset();
    return 0;
  }
  return 1;
}

void TProgram::Reset()
{
  pos_y = 0;
  e1 = 0;
  e3 = 0;
  cur_rule = 0;
  state -> Update(-1, -1, "", "");
  cur_step = 0;
  Update();
}

void TProgram::ClearBreakPoints()
{
  for (int i = 0; i < MAX_LINES; i++)
    lines[i].is_break = 0;
  Update();
}

void TProgram::UpdateFilename()
{
  char buf[40];
  char *t = filename;
  strcpy(buf, " Program ");
  while (*t)
  {
    *t = toupper(*t);
    t++;
  }
  strcat(buf, filename);
  strcat(buf, " ");
  fill(11, 5, 23, 1, '', PROGRAM_COLOR);
  out_string(22 - strlen(buf) / 2, 5, buf, PROGRAM_TITLECOLOR);
}

void TProgram::Save()
{
  if (InputBox("Save program", "Enter file name:", 13, filename, filename, 0))
    return;
  FILE *out = fopen(filename, "wb");

  if (!out)
  {
    MessageBox("Error", "Cannot write file\n", 0);
    return;
  }

  fwrite((char *)lines, sizeof(lines), 1, out);

  fclose(out);
  UpdateFilename();
}

void TProgram::Load()
{
  if (InputBox("Load program", "Enter file name:", 13, "*.MAK", filename, 0))
    return;
  FILE *in = fopen(filename, "rb");
  int t;

  if (!in)
  {
    MessageBox("Error", "Cannot open file\n", 0);
    return;
  }

  fread((char *)lines, sizeof(lines), 1, in);

  fclose(in);
  Update();
  UpdateFilename();
}


#endif
