#ifndef _TYPE_H_
#define _TYPE_H_
#include <stdio.h>
#include <conio.h>
#include "defs.h"
#include "video.h"
#include "utils.h"
#include "clock.h"

class TTape {
    int pos;
    int ed_pos;
    void Draw();
    char filename[40];
  public:
    char data[1000];
    TTape();
    ~TTape();
    void Clear();
    void Edit(char, char);
    void Save();
    void Load();
    int Search(char *);
    void Insert(int, char *);
    void Delete(int, int);
    int CheckEmpty();
    void Update();
};

TTape::TTape()
{
  strcpy(filename, "NONAME.TAP");
  Draw();
  Clear();
  gotoxy(39, 22);
}

TTape::~TTape()
{
}

void TTape::Draw()
{
  fill(5, 19, 67, 1, '', TAPE_COLOR);
  fill(5, 21, 67, 1, '', TAPE_COLOR);
  fill(5, 20, 67, 1, ' ', TAPE_COLOR);
  out_char(4, 19, '', TAPE_COLOR);
  out_char(72, 19, '', TAPE_COLOR);
  out_char(4, 21, '', TAPE_COLOR);
  out_char(72, 21, '', TAPE_COLOR);
  out_char(4, 20, '', TAPE_COLOR);
  out_char(72, 20, '', TAPE_COLOR);
  set_attr(6, 22, 69, 1, SHADOW_COLOR);
  set_attr(73, 20, 2, 3, SHADOW_COLOR);
  Update();
}

void TTape::Update()
{
  for (int i = 0; i < 66; i++)
  {
    if (i + pos < strlen(data))
      out_char(5 + i, 20, data[pos + i], TAPE_CHARCOLOR);
    else
      out_char(5 + i, 20, ' ', TAPE_CHARCOLOR);
  }
}

void TTape::Clear()
{
  int i;
  pos = 0;
  ed_pos = 0;
  data[0] = 0;
  Update();
}

void TTape::Edit(char command, char key)
{
  switch (command)
  {
    case 75: if (ed_pos > 0)
               ed_pos--;
             else
               if (pos)
                 pos--;
             break;
    case 77: if (ed_pos + pos < strlen(data))
             {
               if (ed_pos < 66)
                 ed_pos++;
               else
                 pos++;
             }
             break;
    case 83: if (ed_pos + pos < strlen(data))
               Delete(ed_pos + pos, 1);
             break;
    case 71: pos = 0; ed_pos = 0; break;
    case 79: if (strlen(data) > 66)
             {
               pos = strlen(data) - 66;
               ed_pos = 66;
             }
             else
               ed_pos = strlen(data);
  }
  if ((key == 8) && (ed_pos + pos > 0))
  {
    Delete(ed_pos + pos - 1, 1);
    if (ed_pos)
      ed_pos--;
    else
      pos--;
  }
  if (key >= 0x20)
  {
    if (strlen(data) + 1 <= 999)
    {
      char buf[2];
      buf[0] = key;
      buf[1] = 0;
      Insert(pos + ed_pos, buf);
      if (ed_pos < 66)
        ed_pos++;
      else
        pos++;
    }
  }

  gotoxy(ed_pos + 6, 21);
  Update();
}

int TTape::Search(char *what)
{
  char *t;
  if (!strcmp(data, what))
    return 0;
  else
    t = strstr(data, what);
  if (t == NULL) return -1;
  return t - data;
}

void TTape::Insert(int pos, char *what)
{
  int i = 0;
  i = strlen(data);
  if (i + strlen(what) > 999) return;
  while (i >= pos)
  {
    data[i + strlen(what)] = data[i];
    i--;
  }
  for (int j = 0; j < strlen(what); j++)
    data[j + pos] = what[j];
}

void TTape::Delete(int pos, int count)
{
  int n = strlen(data);
  for (int i = pos; i < n - count + 1; i++)
    data[i] = data[i + count];
}

int TTape::CheckEmpty()
{
  for (int i = 0; i < 1000; i++)
    if (data[i] != ' ') return 0;
  MessageBox("Warning", "Tape is empty\n", 0);
  return 1;
}

void TTape::Save()
{
  if (InputBox("Save tape", "Enter file name:", 13, filename, filename, 0))
    return;
  FILE *out = fopen(filename, "wb");

  if (!out)
  {
    MessageBox("Error", "Cannot write file\n", 0);
    return;
  }

  fwrite((char *)&pos, 2, 1, out);
  fwrite(data, 1000, 1, out);

  fclose(out);
}

void TTape::Load()
{
  if (InputBox("Load tape", "Enter file name:", 13, "*.TAP", filename, 0))
    return;
  FILE *in = fopen(filename, "rb");

  if (!in)
  {
    MessageBox("Error", "Cannot open file\n", 0);
    return;
  }

  fread((char *)&pos, 2, 1, in);
  fread(data, 1000, 1, in);

  fclose(in);
  Update();
}


#endif
